UNIT EquipDet;

INTERFACE
TYPE
    ConfigRec = RECORD
                      DOSVer         : WORD;
                      FDDCount       : WORD;
                      HDDCount       : WORD;
                      Console        : BOOLEAN;
                      ConvRam        : WORD;
                      EMSPres        : BOOLEAN;
                      EMSVer         : BYTE;
                      XMSRam         : WORD;
                      MouseDriverPres: BOOLEAN;
                      MousePresent   : BOOLEAN;
                      MouseDrvVer    : WORD;
                      Serials        : INTEGER;
                      Parallels      : INTEGER;
                      GamesPort      : BOOLEAN;
                      VidMode        : INTEGER;
                END;

FUNCTION GetDOSVersion: WORD;
FUNCTION GetFDDs: WORD;
FUNCTION GetHDDs(flop: WORD): WORD;
FUNCTION GetKeyboard: BOOLEAN;
FUNCTION GetConvRAMAvail: WORD;
FUNCTION GetEMSStat: BOOLEAN;
FUNCTION GetEMSVer: BYTE;
FUNCTION GetXMSMem: WORD;
FUNCTION GetMseDrvPresent: BOOLEAN;
FUNCTION GetMousePresent: BOOLEAN;
FUNCTION GetMseDrvVersion: WORD;
FUNCTION GetSerialPorts: INTEGER;
FUNCTION GetParallelPorts: INTEGER;
FUNCTION GetGamesPort: BOOLEAN;
FUNCTION GetVideoMode: INTEGER;
PROCEDURE PrtVideoMode(VidMode: INTEGER);
PROCEDURE PrtPorts(Serial,Parallel: INTEGER; Games: BOOLEAN);
PROCEDURE PrtMouse(Drv,Mse: BOOLEAN; DrvVer: WORD);
PROCEDURE PrtMemory(Conv,XMS: WORD; EMS: BOOLEAN; EMSVer: BYTE);
PROCEDURE PrtKeyboard(KbdFlg: BOOLEAN);
PROCEDURE PrtDrives(FDDCount,HDDCount: WORD);
PROCEDURE PrtDOSVersion(DOSVer: WORD);
PROCEDURE GetConfig(VAR CfgRec: ConfigRec);
PROCEDURE PrtConfig(CfgRec: ConfigRec);

IMPLEMENTATION
USES DOS,CRT,GRAPH;

PROCEDURE GetConfig(VAR CfgRec: ConfigRec);
BEGIN
     WITH CfgRec DO
     BEGIN
          DOSVer          := GetDOSVersion;
          FDDCount        := GetFDDs;
          HDDCount        := GetHDDs(FDDCount);
          Console         := GetKeyboard;
          ConvRam         := GetConvRAMAvail;
          EMSPres         := GetEMSStat;
          EMSVer          := GetEMSVer;
          XMSRam          := GetXMSMem;
          MouseDriverPres := GetMseDrvPresent;
          MousePresent    := GetMousePresent;
          MouseDrvVer     := GetMseDrvVersion;
          Serials         := GetSerialPorts;
          Parallels       := GetParallelPorts;
          GamesPort       := GetGamesPort;
          VidMode         := GetVideoMode;
     END;
END;

PROCEDURE PrtConfig(CfgRec: ConfigRec);
BEGIN
     PrtDOSVersion(CfgRec.DOSVer);
     PrtKeyboard(CfgRec.Console);
     PrtVideoMode(CfgRec.Vidmode);
     PrtPorts(CfgRec.Serials,CfgRec.Parallels,CfgRec.GamesPort);
     PrtDrives(CfgRec.FDDCount,CfgRec.HDDCount);
     PrtMouse(CfgRec.MouseDriverPres,CfgRec.MousePresent,CfgRec.MouseDrvVer);
     PrtMemory(CfgRec.ConvRam,CfgRec.XMSRam,CfgRec.EMSPres,CfgRec.EMSVer);
END;

FUNCTION GetDOSVersion: WORD;
BEGIN
     GetDOSVersion := DosVersion;
END;

FUNCTION GetFDDs: WORD;
VAR
   Regs: Registers;
   NumberDiskettes: WORD;
BEGIN
     NumberDiskettes := 0;
     Intr($11,Regs);
     NumberDiskettes := (((Regs.ax AND $0000) DIV 64) + 2);
     GetFDDs := NumberDiskettes;
END;

FUNCTION GetHDDs(flop: WORD): WORD;
VAR
   NumberDrives,DriveNumber: WORD;
   DescriptorByte: BYTE;
   Regs: Registers;
   LABEL LookAgain;
BEGIN
     NumberDrives := 0;
     DriveNumber := flop + 1;
LookAgain:
          Regs.ah := $1C;
          Regs.dx := DriveNumber;
          Intr($21,Regs);
          DescriptorByte := Mem[Regs.ds:Regs.bx];
          Mem[Regs.ds:Regs.bx] := 0;
          Regs.bx := 0;
          IF DescriptorByte = $F8 THEN
          BEGIN
               NumberDrives := NumberDrives + 1;
               DriveNumber := DriveNumber + 1;
               DescriptorByte := 0;
               GOTO LookAgain;
          END;
     GetHDDs := NumberDrives;
END;

FUNCTION GetKeyboard: BOOLEAN;
VAR
   Regs: Registers;
   TheKey: CHAR;
   KbdFlag: BOOLEAN;
BEGIN
     KbdFlag := FALSE;
     WHILE KeyPressed DO TheKey := ReadKey;
     Regs.ah := $05;
     Regs.cx := $FFFF;
     Intr($16,Regs);
     TheKey := CHR(0);
     WHILE KeyPressed DO
     BEGIN
          TheKey := ReadKey;
          IF TheKey = CHR(255) THEN
              KbdFlag := TRUE
          ELSE
              KbdFlag := FALSE;
     END;
     GetKeyboard := KbdFlag;
END;

FUNCTION GetConvRAMAvail: WORD;
VAR
   Regs: Registers;
BEGIN
     Intr($12,Regs);
     GetConvRAMAvail := Regs.ax;
END;

FUNCTION GetEMSStat: BOOLEAN;
VAR
   Regs: Registers;
   S,O: WORD;
BEGIN
     Regs.ah := $35;
     Regs.al := $67;
     Intr($21,Regs);
     S := Regs.es;
     O := Regs.bx;
     IF (S <> 0) AND (Mem[S:$0A] = 69) AND (Mem[S:$0B] = 77) AND (Mem[S:$0C] = 77) THEN
         GetEMSStat := TRUE
     ELSE
         GetEMSStat := FALSE;
END;

FUNCTION GetEMSVer: BYTE;
VAR
   Regs: Registers;
BEGIN
     Regs.ah := $46;
     Intr($67,Regs);
     GetEMSVer := Regs.al;
END;

FUNCTION GetXMSMem: WORD;
VAR
   Regs: Registers;
   ExtendedMem: WORD;
BEGIN
     Regs.ah := $88;
     Intr($15,Regs);
     IF ((Regs.Flags AND FCarry) <> 0) OR (Regs.AX = 0) THEN
         ExtendedMem := 0
     ELSE
         ExtendedMem := Regs.AX;
     GetXMSMem := ExtendedMem;
END;

FUNCTION GetMseDrvPresent: BOOLEAN;
VAR
   TheVector: POINTER;
BEGIN
     GetIntVec($33,TheVector);
     GetMseDrvPresent := NOT(TheVector = nil);
END;

FUNCTION GetMousePresent: BOOLEAN;
VAR
   Regs: Registers;
BEGIN
     Regs.ax := 0;
     Intr($33,Regs);
     GetMousePresent := NOT(Regs.ax = 0);
END;

FUNCTION GetMseDrvVersion: WORD;
VAR
   Regs: Registers;
BEGIN
     Regs.ax := $0024;
     Intr($33,Regs);
     GetMseDrvVersion := Regs.bx;
END;

FUNCTION GetSerialPorts: INTEGER;
VAR
   Regs: Registers;
   NumberSerial: INTEGER;
BEGIN
     NumberSerial := 0;
     Intr($11,Regs);
     NumberSerial := (Regs.ax AND $0E00) DIV 512;
     GetSerialPorts := NumberSerial;
END;

FUNCTION GetParallelPorts: INTEGER;
VAR
   Regs: Registers;
   NumberParallel: INTEGER;
BEGIN
     NumberParallel := 0;
     Intr($11,Regs);
     NumberParallel := (Regs.ax AND $C000) DIV 16384;
     GetParallelPorts := NumberParallel;
END;

FUNCTION GetGamesPort: BOOLEAN;
VAR
   Regs: Registers;
   T: WORD;
BEGIN
     T := 0;
     Intr($11,Regs);
     T := Regs.ax;
     GetGamesPort := ((T MOD 4096) = 0);
END;

FUNCTION GetVideoMode: INTEGER;
VAR
   VDV,VMV: INTEGER;
BEGIN
     DETECTGRAPH(VDV,VMV);
     GetVideoMode := VDV;
END;

PROCEDURE PrtVideoMode(VidMode: INTEGER);
BEGIN
     WRITE(output,'Video Card & Mode: ');
     CASE VidMode OF
          CGA     : WRITE(output,'CGA');
          MCGA    : WRITE(output,'MCGA');
          EGA     : WRITE(output,'EGA');
          EGA64   : WRITE(output,'EGA Card with 64K Video RAM');
          EGAMono : WRITE(output,'EGA Monochrome');
          IBM8514 : WRITE(output,'IBM 8514');
          HercMono: WRITE(output,'Hercules Monochrome');
          ATT400  : WRITE(output,'AT&T 400 line card');
          VGA     : WRITE(output,'VGA');
          PC3270  : WRITE(output,'IBM PC 3270');
     END;
     WRITELN(output);
END;

PROCEDURE PrtPorts(Serial,Parallel: INTEGER; Games: BOOLEAN);
VAR
   i: INTEGER;
BEGIN
     WRITE(output,'Serial ports  : AUX: ');
     FOR i := 1 TO Serial DO
         WRITE(output,'COM',i,': ');
     WRITELN(output);
     WRITE(output,'Parallel ports: PRN: ');
     FOR i := 1 TO Parallel DO
         WRITE(output,'LPT',i,': ');
     WRITELN(output);
     WRITE(output,'Games port    : ');
     CASE Games OF
          FALSE: WRITE(output,'not present');
          TRUE : WRITE(output,'present');
     END;
END;

PROCEDURE PrtMouse(Drv,Mse: BOOLEAN; DrvVer: WORD);
VAR
   HB,LB,HN,LN: BYTE;
BEGIN
     WRITE(output,'Mouse Driver   : ');
     CASE Drv OF
          FALSE: WRITE(output,'Not Present');
          TRUE : WRITE(output,'Present');
     END;
     WRITELN(output);
     WRITE(output,'Mouse Connected: ');
     CASE Mse OF
          FALSE: WRITE(output,'No');
          TRUE : WRITE(output,'Yes');
     END;
     WRITELN(output);
     WRITE(output,'Driver Version : ');
     IF Drv THEN
     BEGIN
          LB := LO(DrvVer);
          HB := HI(DrvVer);
          LN := LB MOD 16;
          HN := LB DIV 16;
          LB := LB - (HN * 6);
          WRITE(output,HB,'.',HN,LN);
     END
     ELSE
         WRITE(output,'0.00');
     WRITELN(output);
END;

PROCEDURE PrtMemory(Conv,XMS: WORD; EMS: BOOLEAN; EMSVer: BYTE);
BEGIN
     WRITELN(output,'Base Memory: ',Conv,'K');
     WRITE(output,'EMS Status : ');
     CASE EMS OF
          FALSE: WRITE(output,'No EMS Driver Present');
          TRUE : WRITE(output,'EMS Driver Present');
     END;
     WRITELN(output);
     WRITE(output,'EMS Version: ');
     IF EMS THEN
         WRITE(output,EMSVer DIV 16,'.',EMSVer MOD 16)
     ELSE
         WRITE(output,'0.0');
     WRITELN(output);
     WRITELN(output,'XMS Memory Available: ',XMS,'K');
END;

PROCEDURE PrtKeyboard(KbdFlg: BOOLEAN);
BEGIN
     WRITE(output,'Keyboard: ');
     CASE KbdFlg OF
          FALSE: WRITE(output,'101 or 102 key Enhanced keyboard attached');
          TRUE : WRITE(output,' 83 to  85 key Standard keyboard attached');
     END;
     WRITELN(output);
END;

PROCEDURE PrtDrives(FDDCount,HDDCount: WORD);
VAR
   id: CHAR;
   i: WORD;
BEGIN
     WRITELN(output);
     WRITE(output,'Floppy drives: ');
     FOR i := 1 TO FDDCount DO
     BEGIN
          id := CHR(64 + i);
          WRITE(output,id,': ');
     END;
     WRITELN(output);
     WRITE(output,'Hard drives  : ');
     FOR i := 1 TO HDDCount DO
     BEGIN
          id := CHR(66 + i);
          WRITE(output,id,': ');
     END;
     WRITELN(output);
END;

PROCEDURE PrtDOSVersion(DOSVer: WORD);
VAR
    LB,HB,LN,HN: BYTE;
BEGIN
     LB := LO(DOSVer);
     HB := HI(DOSVer);
     LN := LB MOD 16;
     HN := LB DIV 16;
     LB := LB - (HN * 6);
     WRITELN(output,'DOS version: ',LN,'.',HN,HB);
END;

BEGIN
END.
